@@ -726,7 +726,7 @@ impl Stack {
 enum PKey {
     None,
     Dyn(Box<dyn Record>),
-    Parent(usize),
+    Copy(Vec<u8>),
 }
 
 /// PageList is used to implement repacking of child pages ( REPACKFILE builtin function ).
@@ -738,7 +738,7 @@ struct PageList {
     count: usize,
 }
 
-const TRACE_PACK: bool = true;
+const TRACE_PACK: bool = false;
 
 /// Limit on how many pages to free in one transaction.
 const REPACK_LIMIT: i64 = 10;
@@ -754,10 +754,7 @@ impl PageList {
             let cpnum = pnums.pop().unwrap();
             file.publish_page(cpnum, cp);
             match key {
-                PKey::Parent(x) => {
-                    p.set_child_page(x, cpnum);
-                    np.append_from(p, x);
-                }
+                PKey::Copy(b) => np.append_page_copy(&b, cpnum),
                 PKey::Dyn(key) => np.append_page(&*key, cpnum),
                 PKey::None => np.first_page = cpnum,
             }
@@ -799,20 +796,17 @@ impl PageList {
         p.pnum = u64::MAX;
 
         if p.level > 0
-        // Need to save first_page somehow.
+        // Need to save first_page. Append the parent key, then fixup the page number.
         {
             if let Some(pp) = par {
-                self.append_one(db, pp, px, r); // Use parent key ( even thouogh it will not have correct page ).
-                let ap = &mut self.list[self.cur].0;
-                let x = ap.count;
-                if x == 0 {
-                    ap.first_page = p.first_page;
-                } else {
-                    ap.set_child_page(x, p.first_page);
-                }
-            } else {
-                let ap = &mut self.list[self.cur].0;
+                self.append_one(db, pp, px, r);
+            }
+            let ap = &mut self.list[self.cur].0;
+            let x = ap.count;
+            if x == 0 {
                 ap.first_page = p.first_page;
+            } else {
+                ap.set_child_page(x, p.first_page);
             }
         }
         self.append(db, p, p.root, r);
@@ -835,7 +829,7 @@ impl PageList {
                 let key = p.get_key(db, x, r);
                 PKey::Dyn(key)
             } else {
-                PKey::Parent(x)
+                PKey::Copy( p.copy(x) )
             };
             self.list.push((p.new_page(), key));
             self.cur += 1;
