@@ -165,20 +165,18 @@ fn get_table0(db: &DB, name: &ObjRef) -> Option<(i64, i64, i64)> {
     None
 }
 
-pub fn get_index(db: &DB, tname: &ObjRef, iname: &String) -> (TablePtr, usize, u64) {
+pub fn get_index(db: &DB, tname: &ObjRef, iname: &str) -> (TablePtr, usize, u64) {
     if let Some(t) = get_table(db, tname) {
         // Loop through indexes. Columns are Root, Table, Name.
         let ixt = &db.sys_index;
-        let mut ix = 0;
         let key = Value::Int(t.id);
-        for (pp, off) in ixt.scan_key(db, key, 0) {
+        for (ix, (pp, off)) in ixt.scan_key(db, key, 0).enumerate() {
             let p = &pp.borrow();
             let a = ixt.access(p, off);
-            if &a.str(db, 2) == iname {
+            if a.str(db, 2) == iname {
                 let id = a.id();
                 return (t, ix, id);
             }
-            ix += 1;
         }
         panic!("index {} not found", iname);
     } else {
@@ -305,3 +303,15 @@ pub fn save_id_gen(db: &DB, id: u64, val: i64) {
     wa.set_int(3, val);
     t.file.set_dirty(p, &pp);
 }
+
+/// Update root page for table ( for ALTER TABLE ).
+pub fn set_root(db: &DB, id: i64, new_root: u64) {
+    let id = id as u64;
+    let t = &db.sys_table;
+    let (pp, off) = t.id_get(db, id).unwrap();
+    let p = &mut pp.borrow_mut();
+    let mut wa = t.write_access(p, off);
+    debug_assert!(wa.id() == id);
+    wa.set_int(0, new_root as i64);
+    t.file.set_dirty(p, &pp);
+}
